<?php

require 'classes/Quest.php';

class SignController{

    protected function getMysql($account_id)
    {
        $mysql_conf = getMysqlConfig(crc32($account_id));
        $conn = new phpcommon\Mysql(array(
            'host' => $mysql_conf['host'],
            'port' => $mysql_conf['port'],
            'user' => $mysql_conf['user'],
            'passwd' => $mysql_conf['passwd'],
            'dbname' => 'gamedb2001_' . $mysql_conf['instance_id']
        ));
        return $conn;
    }

    protected function getSign($sign_id)
    {
        $g_conf_sign_cluster = require('../res/game2001api.signDaily.cluster.php');
        $sign_conf = getSignConfig($g_conf_sign_cluster, $sign_id);
        $s = array(
            'sign_id' => $sign_conf['sign_id'],
            'condition' => $sign_conf['condition'],
            'item_id' => $sign_conf['item_id'],
            'num' => $sign_conf['num'],
        );
        return $s;
    }

    protected function getActivityRewardConfig($activityReward_id)
    {
        $g_conf_activityReward_cluster = require('../res/game2001api.activityReward.cluster.php');
        $activityReward_conf = getActivityRewardConfig($g_conf_activityReward_cluster, $activityReward_id);
        $act = array(
            'id' => $activityReward_conf['id'],
            'activity_id' => $activityReward_conf['activity_id'],
            'condition' => $activityReward_conf['condition'],
            'parameter' => $activityReward_conf['parameter'],
            'start_end_time' => $activityReward_conf['start_end_time'],
            'activity_reward' => $activityReward_conf['activity_reward'],
            'exchange_num' => $activityReward_conf['exchange_num'],
            'exchange_item' => $activityReward_conf['exchange_item']
        );
        return $act;
    }

    protected function getExplode($string)
    {
        $delim = "|";
        $drop_multiply = explode($delim, $string);
        $delim1 = ":";
        $arr = array();
        for ($i = 0; $i < count($drop_multiply); $i++) {
            $mul = explode($delim1, $drop_multiply[$i]);
            array_push($arr, $mul);
        }
        return $arr;
    }

    protected function getParameter($para_id)
    {
        $g_conf_para_cluster = require('../res/game2001api.parameter.cluster.php');
        $para_conf = getParameterConfig($g_conf_para_cluster, $para_id);
        $p = array(
            'id' => $para_conf['id'],
            'param_name' => $para_conf['param_name'],
            'param_value' => $para_conf['param_value'],
        );
        return $p;
    }

    public function signInfo()
    {
        $account_id = $_REQUEST['account_id'];
        //登录校验
        $login = loginVerify($account_id, $_REQUEST['session_id']);
        if (!$login) {
            phpcommon\sendError(ERR_USER_BASE + 1, 'session无效');
            return;
        }
        $conn = $this->getMysql($account_id);
        $sign_days = 0;
        $signable = 0;
        if (!$conn) {
            phpcommon\sendError(ERR_USER_BASE + 1, '没有这个玩家');
            return;
        }
        $row1 = $conn->execQueryOne('SELECT * FROM user WHERE accountid=:accountid;',
                                   array(
                                       ':accountid' => $account_id
                                   ));

        $row = $conn->execQueryOne('SELECT * FROM sign WHERE accountid=:accountid;',
                                   array(
                                       ':accountid' => $account_id
                                   ));
        //触发活动任务
        $act = $this->getActivityRewardConfig(QUEST_ACTIVITY_LOGIN);
        $array = $this->getExplode($act['start_end_time']);
        $nowTime = phpcommon\getdayseconds(time());
        $start_time = $array[0][0] * 3600 + $array[0][1] * 60 + $nowTime;
        $end_time = $array[1][0] * 3600 + $array[1][1] * 60 + $nowTime;
        if (!$row) {
            $ret = $conn->execScript('INSERT INTO sign(accountid, sign_days, signable, sign_time, create_time, modify_time) ' .
                                     ' VALUES(:accountid, :sign_days, :signable, :sign_time, :create_time, :modify_time) ' .
                                     ' ON DUPLICATE KEY UPDATE accountid=:accountid, sign_days=:sign_days, signable=:signable, sign_time=:sign_time, modify_time=:modify_time;',
                                     array(
                                         ':accountid' => $account_id,
                                         ':sign_days' => 1,
                                         ':signable' => 0,
                                         ':sign_time' => time(),
                                         ':create_time' => time(),
                                         ':modify_time' => time()
                                     ));
            if (!$ret) {
                die();
                return;
            }
            $sign_days = 1;
            $quest = new classes\Quest();
            $quest->triggerQuest(QUEST_DAY_LOGIN, 1, 1, $account_id);
            $quest->triggerQuest(QUEST_SUM_LOGIN, 2, 1, $account_id);
            if (time() >= $start_time && time() < $end_time) {
                $quest->triggerQuest(QUEST_ACTIVITY_LOGIN, 3, 1, $account_id);
            }
        } else {
            if ($nowTime - phpcommon\getdayseconds($row['sign_time']) > 0) {
                $sign_days = $row['sign_days'] + 1;
                $signable = 0;
                $ret = $conn->execScript('UPDATE sign SET sign_days=:sign_days, sign_time=:sign_time, signable=:signable, modify_time=:modify_time ' .
                                         ' WHERE accountid=:accountid;',
                                         array(
                                             ':accountid' => $account_id,
                                             ':sign_days' => $sign_days,
                                             ':signable' => $signable,
                                             ':sign_time' => time(),
                                             ':modify_time' => time()
                                         ));
                if (!$ret) {
                    die();
                    return;
                }
                //刷新每日任务和活动任务
                $rowCount = $conn->execQueryRowCount('SELECT * FROM quest WHERE accountid=:accountid;',
                                                     array(
                                                         ':accountid' => $account_id,
                                                     ));
                if ($rowCount != 0) {
                    for ($i = 0; $i < $rowCount; $i++) {
                        $ret = $conn->execScript('UPDATE quest SET quest_state=:quest_state, quest_num=:quest_num, modify_time=:modify_time ' .
                                                 ' WHERE accountid=:accountid AND quest_type=:quest_type1 OR quest_type=:quest_type2;',
                                                 array(
                                                     ':accountid' => $account_id,
                                                     ':quest_num' => 0,
                                                     ':quest_type1' => 1,
                                                     ':quest_type2' => 3,
                                                     ':quest_state' => 0,
                                                     ':modify_time' => time()
                                                 ));
                    }
                    if (!$ret) {
                        die();
                        return;
                    }
                }
                $quest = new classes\Quest();
                $quest->triggerQuest(QUEST_DAY_LOGIN, 1, 1, $account_id);
                $quest->triggerQuest(QUEST_SUM_LOGIN, 2, 1, $account_id);
                //刷新活动皮肤碎片收集状态
                $rowSkinCount = $conn->execQueryRowCount('SELECT * FROM skin WHERE accountid=:accountid AND skin_type=:skin_type;',
                                                         array(
                                                             ':accountid' => $account_id,
                                                             ':skin_type' => 2
                                                         ));
                if ($rowSkinCount != 0) {
                    for ($i = 0; $i < $rowSkinCount; $i++) {
                        $ret = $conn->execScript('UPDATE skin SET fragment_status=:fragment_status, modify_time=:modify_time ' .
                                                 ' WHERE accountid=:accountid;',
                                                 array(
                                                     ':accountid' => $account_id,
                                                     ':fragment_status' => 0,
                                                     ':modify_time' => time()
                                                 ));
                        if (!$ret) {
                            die();
                            return;
                        }
                    }
                }
            } else {
                $sign_days = $row['sign_days'];
                $signable = $row['signable'];
                if (time() >= $start_time && time() < $end_time) {
                    $quest = new classes\Quest();
                    $quest->triggerQuest(QUEST_ACTIVITY_LOGIN, 3, 1, $account_id);
                }
            }
        }
        echo json_encode(array(
            'errcode' => 0,
            'errmsg' => '',
            'sign_days' => $sign_days,
            'signable' => $signable
        ));
    }

    public function addReward($item_id, $item_num, $account_id)
    {
        $conn = $this->getMysql($account_id);
        if (!$conn) {
            phpcommon\sendError(ERR_USER_BASE + 1, '没有这个玩家');
            return;
        }
        $rowSkin = $conn->execQueryOne('SELECT * FROM skin WHERE accountid=:accountid AND fragment_id=:fragment_id;',
                                       array(
                                           ':accountid' => $account_id,
                                           ':fragment_id' => $item_id
                                       ));
        $ret = $conn->execScript('UPDATE skin SET fragment_num=:fragment_num, modify_time=:modify_time ' .
                                 ' WHERE accountid=:accountid AND fragment_id=:fragment_id;',
                                 array(
                                     ':accountid' => $account_id,
                                     ':fragment_id' => $item_id,
                                     ':fragment_num' => $item_num + $rowSkin['fragment_num'],
                                     ':modify_time' => time()
                                 ));
        if (!$ret) {
            die();
        }
    }

    public function signReward()
    {
        $account_id = $_REQUEST['account_id'];
        $double = $_REQUEST['double'];
        //登录校验
        $login = loginVerify($account_id, $_REQUEST['session_id']);
        if (!$login) {
            phpcommon\sendError(ERR_USER_BASE + 1, 'session无效');
            return;
        }
        $conn = $this->getMysql($account_id);
        if (!$conn) {
            phpcommon\sendError(ERR_USER_BASE + 1, '没有这个玩家');
            return;
        }
        $row = $conn->execQueryOne('SELECT * FROM sign WHERE accountid=:accountid;',
                                   array(
                                       ':accountid' => $account_id
                                   ));
        if (!$row || $row['signable'] == 1) {
            phpcommon\sendError(ERR_USER_BASE + 2, '今日已签到');
            return;
        }

        $ret = $conn->execScript('UPDATE sign SET sign_time=:sign_time, signable=1, modify_time=:modify_time ' .
                                 ' WHERE accountid=:accountid;',
                                 array(
                                     ':accountid' => $account_id,
                                     ':sign_time' => time(),
                                     ':modify_time' => time()
                                 ));
        if (!$ret) {
            die();
            return;
        }
        //获得奖励
        $g_conf_sign_cluster = require('../res/game2001api.signDaily.cluster.php');
        $sign_days = $row['sign_days'] % count($g_conf_sign_cluster);
        if ($sign_days == 0) {
            $sign_days = count($g_conf_sign_cluster);
        }
        $s = $this->getSign($sign_days + 90000);
        $item_id = $s['item_id'];
        $num = $s['num'];
        if ($double == 1) {
            $p_gold = $this->getParameter(DOUBLE_SIGN_IN);
            $num = $num * $p_gold['param_value'];
        }
        if ($item_id == 10001) {
            $row2 = $conn->execQueryOne('SELECT * FROM user WHERE accountid=:accountid;',
                                        array(
                                            ':accountid' => $account_id
                                        ));
            $ret = $conn->execScript('UPDATE user SET coin_num=:coin_num, modify_time=:modify_time WHERE accountid=:accountid;',
                                     array(
                                         ':accountid' => $account_id,
                                         ':coin_num' => $num + $row2['coin_num'],
                                         ':modify_time' => time()
                                     ));
            if (!$ret) {
                die();
                return;
            }
        } else if($item_id > 13000 && $item_id < 14000){
            $this->addReward($item_id, $num, $account_id);
        }

        echo json_encode(array(
            'errcode' => 0,
            'errmsg' => '',
        ));
    }
}
?>
